-- Avoiding corners with bump.lua.
-- Copyright © 2016 Pedro Gimeno Fortea.

-- This work is free. You can redistribute it and/or modify it under the
-- terms of the Do What The Fuck You Want To Public License, Version 2,
-- as published by Sam Hocevar. See http://www.wtfpl.net/ for more details.


local bump = require 'bump'

local world = bump.newWorld(2)

-- Player initial x,y; player width/height
local px, py, pw, ph = 6, 6, 0.75, 0.75

local tilesx, tilesy = 64, 64

local margin = 20/tilesx -- correction is applied at 20 pixels from the corner

-- Player velocity (in cells/s)
local pvx, pvy = 2, 2

do
  local _ = true
  local X = false
  walkable = {{X,X,X,X,X,X,X,X},
              {X,_,_,_,_,_,_,X},
              {X,_,X,X,_,X,_,X},
              {X,_,X,_,X,_,_,X},
              {X,_,_,_,_,_,_,X},
              {X,_,_,X,_,_,_,X},
              {X,_,_,_,_,_,_,X},
              {X,X,X,X,X,X,X,X}}

  -- For demo purposes, we make our map match the walkable array.
  map = {}
  for y = 1, #walkable do
    map[y] = {}
    for x = 1, #walkable[y] do
      map[y][x] = walkable[y][x] and 0 or 1
    end
  end
end

local lg = love.graphics


function love.load(args)

  -- Add to bump
  for y = 1, #walkable do
    for x = 1, #walkable[y] do
      if not walkable[y][x] then
        world:add("wall"..x..","..y, x, y, 1, 1)
      end
    end
  end

  world:add("player", px, py, pw, ph)
end


-- this is array in order up, down, left, right
local controls = {love.keyboard.isScancodeDown("w") and 1 or 0,
                  love.keyboard.isScancodeDown("s") and 1 or 0,
                  love.keyboard.isScancodeDown("a") and 1 or 0,
                  love.keyboard.isScancodeDown("d") and 1 or 0}

local coll1, coll2, coll3, coll4 = false, false, false, false


function love.keypressed(k, s)
  if s == "w" then controls[1] = 1 end
  if s == "s" then controls[2] = 1 end
  if s == "a" then controls[3] = 1 end
  if s == "d" then controls[4] = 1 end

  if k == "escape" then love.event.quit() end
end

function love.keyreleased(_, s)
  if s == "w" then controls[1] = 0 end
  if s == "s" then controls[2] = 0 end
  if s == "a" then controls[3] = 0 end
  if s == "d" then controls[4] = 0 end
end


function love.update(dt)
  -- We don't allow a dt that makes our player walk a half player body or more;
  -- if the user's computer is that slow, too bad.
  if pvx*dt >= pw*0.5 then
    dt = pw*0.5/pvx-0.0000001
  end
  if pvy*dt >= ph*0.5 then
    dt = ph*0.5/pvy-0.0000001
  end

  local newpx, newpy = px, py

  -- Apply movement to the player
  newpy = newpy - controls[1] * pvy * dt + controls[2] * pvy * dt
  newpx = newpx - controls[3] * pvx * dt + controls[4] * pvx * dt

  if controls[1] + controls[2] + controls[3] + controls[4] == 1 then

    -- Only one control key pressed - which one?
    local dir = controls[1] + 2*controls[2] + 3*controls[3] + 4*controls[4]


    -- This is the actual corner detection and correction
    local coll, n
    coll, n = world:queryRect(newpx, newpy, pw, ph, function(item) return item ~= "player" end)

    if n > 0 then -- player collides at that point
      if dir == 1 or dir == 2 then
        coll, n = world:queryRect(newpx + margin, newpy, pw - margin, ph, function(item) return item ~= "player" end)
        if n == 0 then
          newpx = newpx + pvx * dt
        else
          coll, n = world:queryRect(newpx, newpy, pw - margin, ph, function(item) return item ~= "player" end)
          if n == 0 then
            newpx = newpx - pvx * dt
          end
        end
      else
        coll, n = world:queryRect(newpx, newpy + margin, pw, ph - margin, function(item) return item ~= "player" end)
        if n == 0 then
          newpy = newpy + pvy * dt -- walk diagonally
        else
          coll, n = world:queryRect(newpx, newpy, pw, ph - margin, function(item) return item ~= "player" end)
          if n == 0 then
            newpy = newpy - pvy * dt
          end
        end
      end
    end
  end

  -- Update player's position
  local coll, n
  px, py = world:move("player", newpx, newpy)
end


local viewcoll


function love.draw()

  for y, row in ipairs(map) do
    for x, tile in ipairs(row) do
      if tile == 1 then
        lg.setColor(224, 224, 224)
        lg.rectangle("fill", (x-1)*tilesx, (y-1)*tilesy, tilesx, tilesy)
        lg.setColor(255, 255, 255)
        lg.rectangle("line", (x-1)*tilesx + 0.5, (y-1)*tilesy + 0.5, tilesx-1, tilesy-1)
      end
    end
  end

  local rpx, rpy = (px-1)*tilesx, (py-1)*tilesy
  lg.setColor(64, 128, 64)
  lg.rectangle("fill", rpx, rpy, tilesx*pw, tilesy*ph)
  lg.setColor(144, 255, 144)
  lg.rectangle("line", rpx+0.5, rpy+0.5, tilesx*pw-1, tilesy*ph-1)
end
